//********************************************************************
//
//	Copyright 2000 Trancer Software Inc.
//
//	File Name:	ObjectDump.cpp
//
//	Description:
//		This is the main file for building a FS Design Studio plugin.
//		There are two parts to this file.  Part one was created by the
//		MFC AppWizard (dll).  The second part is the ExecutePlugin
//		function.  The ExecutePlugin function is a required function
//		that is called by the FS Design Studio 'Execute Plugin' menuitem.
//		
//********************************************************************

#include "stdafx.h"
#include <afxdllx.h>
#include "Plugin.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Required define of a CFSObject
CFSObject*	piobject = NULL;

// Generated by MFC AppWizard (dll)
static AFX_EXTENSION_MODULE ObjectDumpDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("OBJECTDUMP.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(ObjectDumpDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(ObjectDumpDLL);

	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("OBJECTDUMP.DLL Terminating!\n");
		// Terminate the library before destructors are called
		AfxTermExtensionModule(ObjectDumpDLL);
	}
	return 1;   // ok
}


//********************************************************************
//
//	Function Name:	ExecutePlugin
//
//	Description:
//		This function is called from the FS Design Studio 'Execute Plugin'
//		menuitem.  It is mainly provided as a template for users to create
//		thier own plugins.
//		This reads the object and prints out a dumpfile of an object's parts.
//
//	Parameters:
//		None.
//
//	Returns:
//		int		Three possible values:	
//			PLUGIN_FAILED	Failed don't reload object
//			PLUGIN_RELOAD	Success and Reload plugin modified object to FSDS
//			PLUGIN_SKIP		Success and Do Not Reload object to FSDS
//		
//********************************************************************

int ExecutePlugin()
{
	piobject = new CFSObject;

       
	// Read a fsc temp file and store the object in piobject
	// This is a required call to load a copy of the object that is 
	// currently being edited in FS Design Studio
	LoadFSC(piobject);

	CString cs = "Object Dump\r\n";
	CString cstemp;
	CString txtfilename = "";
	CString ptxtfilename = "";
	int partnum,polynum,txtnum;
	int pointcount = 0;
	int polycount = 0;

	// For each part in the object do the following
	for(partnum = 0; partnum < piobject->m_PartArray.GetSize(); partnum++)
	{
		CObjectPart* part = piobject->m_PartArray[partnum];
		part->SetSurfaceNormals();
		
		// Dump part header
		cstemp.Format("\r\nPart %s %d:\r\n", part->m_Name, partnum);
		cs += cstemp;

		// Dump part axis
		cstemp.Format("    Pos: %f %f %f\r\n    Rot: %f %f %f\r\n",
			part->m_Axis.Location.X,part->m_Axis.Location.Y,
			part->m_Axis.Location.Z,
			part->m_Axis.Rotation.X,part->m_Axis.Rotation.Y,
			part->m_Axis.Rotation.Z);
		cs += cstemp;

		// Dump part vertices
		cstemp.Format("   Total Part Vertices: %s: %d\r\n", part->m_Name, part->m_VertexArray.GetSize());
		cs += cstemp;
		cs += "\r\n";

		// Dump part polygons
		cs += "  Polygons:\r\n";
		cstemp.Format("   Total Part Polygons: %s: %d\r\n", part->m_Name, part->m_PolygonArray.GetSize());
		cs += cstemp;

		// Find number of concave polygons
		int concavepolygons = 0;
		for(polynum = 0; polynum < part->m_PolygonArray.GetSize(); polynum++)
		{
			if(part->CheckConcavePolygon(polynum))
				concavepolygons++;
		}
		cstemp.Format("   Total Concave Polygons: %s: %d\r\n", part->m_Name, concavepolygons);
		cs += cstemp;

		cs += "\r\n";
		ptxtfilename = "";
		cs += "  Textures:\r\n";

		polycount += part->m_PolygonArray.GetSize();
		pointcount += part->m_VertexArray.GetSize();
		for(polynum = 0; polynum < part->m_PolygonArray.GetSize(); polynum++)
		{
			CPolygon* poly = part->m_PolygonArray[polynum];

			// Dump polygon textures
			for(txtnum = 0; txtnum < 7; txtnum++)
			{
				txtfilename = poly->m_Texture.GetFilename();
				if (ptxtfilename != txtfilename)
				{
					if(txtfilename.GetLength() > 0)
					{
						cstemp.Format("    %4d: %s\r\n", txtnum, txtfilename);
						cs += cstemp;
					}
				}
				ptxtfilename = txtfilename;
			}

		}

		// Bounding box for sizing
		BoundingBox partBounds;
		part->GetBounds(&partBounds);
		
		// calculate texture scaling factors
		double partWidth = part->CalcWidth();
		double partHeight = part->CalcHeight();
		double partDepth = part->CalcDepth();
		double leftedge = part->LeftMostPoint();
		double frontedge = part->FrontMostPoint();
		double rearedge = part->RearMostPoint();
		double bottomedge = part->BottomMostPoint();

		cs += "\r\n  Part Specifications:\r\n";
		cstemp.Format("\tWidth:\t\t%12.4f\r\n\tHeight:\t\t%12.4f\r\n\tDepth:\t\t%12.4f\r\n",
			partWidth, partHeight, partDepth);
		cs += cstemp;
		cstemp.Format("\tLeft Edge:\t%12.4f\r\n\tFront Edge:\t%12.4f\r\n\tRear Edge:\t%12.4f\r\n\tBottom Edge:%12.4f\r\n",
			leftedge, frontedge, rearedge, bottomedge);
		cs += cstemp;

	}

	cs += "\r\nObject Specifications:\r\n";
	cstemp.Format("\tTotal Part Count:\t%12d\r\n\tTotal Polygon Count:%12d\r\n\tTotal Point Count:\t%12d\r\n",
		piobject->m_PartArray.GetSize(), pointcount, polycount);
	cs += cstemp;

	BoundingBox bounds;
	piobject->GetBounds(&bounds);

	cs += "\r\nObject Bounds:\r\n";

	cstemp.Format("\tLargest Extent:\t\t%12.4f\r\n\tMin X:\t\t\t\t%12.4f\r\n\tMax X:\t\t\t\t%12.4f\r\n\tMin Y:\t\t\t\t%12.4f\r\n\tMax Y:\t\t\t\t%12.4f\r\n\tMin Z:\t\t\t\t%12.4f\r\n\tMax Z:\t\t\t\t%12.4f\r\n",
		piobject->GetLargestExtent(), bounds.fMaxX, bounds.fMinX, bounds.fMaxY, bounds.fMinY, bounds.fMaxZ, bounds.fMinZ);
	cs += cstemp;

	CFile f;
	if(f.Open("ObjectDump.txt",CFile::modeWrite | CFile::modeCreate))
	{
		f.Write(cs,cs.GetLength());
		f.Close();
	}
	if(cs.GetLength() > 1024)
		cs = "Data too large for display!\r\n";
	cs += "----------------------\r\nData written to file: ObjectDump.txt\r\n";
	AfxMessageBox(cs);

	// If the plugin modifies the object and you want to send it back to FS Design Studio
	// you'll need to run this function.  If the plugin doesn't modify the object
	// it's not necessary to make this call.
	StoreFSC(piobject);

	delete piobject;

	// Since the object wasn't modified by this plugin just return Success without
	// reloading the object.
	return(PLUGIN_SKIP);
}

