// ObjectPart.h: interface for the CObjectPart class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OBJECTPART_H__9AD62FA3_C1E7_11D2_BE0A_0020781069DC__INCLUDED_)
#define AFX_OBJECTPART_H__9AD62FA3_C1E7_11D2_BE0A_0020781069DC__INCLUDED_

#include <afxtempl.h>
#include "Axis.h"	// Added by ClassView
#include "Vertex.h"	// Added by ClassView
#include "FSBitmap.h"	// Added by ClassView
#include "Vector.h"	// Added by ClassView

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

typedef struct Bounds
{
	CVertex Location;
	double fMinX;
	double fMaxX;
	double fMinY;
	double fMaxY;
	double fMinZ;
	double fMaxZ;
} BoundingBox;

typedef struct TransformCall
{
	BOOL Transformed;
	BOOL UsePartOffset;
	double Xdelta;
	double Ydelta;
	double Zdelta;
	double Xrot;
	USHORT Xvar;
	double Yrot;
	USHORT Yvar;
	double Zrot;
	USHORT Zvar;
} TransformCall;

class CObjectPart  
{
public:
	USHORT m_ZBias;
	TransformCall m_Trans;
	ULONG m_ID;
	ULONG m_ParentID;
	BOOL m_AircraftExternal;
	BOOL m_AircraftInternal;
	int PolygonCount();
	int VertexCount();
	BOOL m_TransparentTexture;
	short m_Color;
	COLORREF m_rgbaColor;
	BOOL IsTransparent();
	BOOL m_CrashDetect;
	void ClearTextures();
	int m_WrapAxis;
	CString m_VarPreset;
	WORD m_VarVal2;
	WORD m_VarVal1;
	int m_VarOp;
	int m_VarID;
	BOOL m_ReferencePart;
	void Cleanup();
	int GetSelectedPointCount();
	BOOL m_Hidden;
	void MakeTriangles(int nPoly);
	BOOL CheckPolygons(BOOL bFix);
	BOOL CheckPolygon(int nPoly);
	BOOL CheckConcavePolygon(int nPoly);
	void DeleteVertex(int nVert);
	int SelectedPolygonCount();
	int SelectedVertexCount();
	int m_RenderType;
	CVector GetVertexNormal(int vertindex);
	void TranslatePoints(double X,double Y,double Z);
	BOOL m_SmoothShaded;
	double CalcDepth();
	double BottomMostPoint();
	double RightMostPoint();
	double RearMostPoint();
	double FrontMostPoint();
	double TopMostPoint();
	double LeftMostPoint();
	double CalcHeight();
	double CalcWidth();
	CString m_Name;
	CString m_PartParentName;
	void MirrorPolygon(CPolygon* poly);
	void FlipPolygon(CPolygon* poly);
	void MirrorAllPolygons(BOOL mirror);
	void MirrorCurrentPolygon();
	void FlipAllPolygons();
	void FlipCurrentPolygon();
	void AddPolygon(CPolygon* poly);
	void AddVertex(CVertex* vertex);
	void CalcPolyCenters();
	double CalcDistance(CVertex vert);
	void Serialize(CArchive& ar);
	void TranslateAxis(double x,double y,double z);
	CVertex FindPolygonCenter(int nPoly);
	void Mirror(int axis);
	void RotateVertices(double x,double y,double z,
		double cx = 0,double cy = 0, double cz = 0);
	void Translate(double x,double y,double z);
	void SetSurfaceNormals();
	CVector GetSurfaceNormal(CPolygon* poly);
	void GetBounds(BoundingBox * bounds);

	BoundingBox m_Bounds;
	CVertex* GetOppositeZVertex(int pnt);
	CVertex* GetOppositeXVertex(int pnt);
	int GetOppositeZIndex(int pnt);
	int GetOppositeXIndex(int pnt);
	int m_CurrentCSPoint;
	int m_CurrentCrossSection;
	int m_CurrentVertex;		// current vertex index
	int m_CurrentPoly;			// current polygon index
	BOOL m_Structured;			// if true, treat as structure
	int m_StructureSegments;	// number of segments in structure
	int m_SegmentPoints;		// number of points in each segment
	int m_StructureAxis;		// segment points encircle this axis
								// (0 = x, 1 = y, 2 = z)
	BOOL m_Selected;			// this part is selected
	//int m_StructureAxis;
	CAxis m_Axis;				// part's axis
	CArray<CPolygon*, CPolygon*&> m_PolygonArray;	// array of object parts
	CArray<CVertex*, CVertex*&> m_VertexArray;	// array of vertices
	CFSBitmap m_Textures[7];

	CObjectPart();
	CObjectPart(const CObjectPart &part);
	virtual ~CObjectPart();

protected:
	double m_Distance;
private:
};

#endif // !defined(AFX_OBJECTPART_H__9AD62FA3_C1E7_11D2_BE0A_0020781069DC__INCLUDED_)
